package ru.dbotthepony.mc.otm.capability;

import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.food.FoodData;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.ICapabilityProvider;
import net.minecraftforge.common.util.INBTSerializable;
import net.minecraftforge.common.util.LazyOptional;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.math.BigDecimal;

public class AndroidCapability implements ICapabilityProvider, IAndroidCapability, INBTSerializable<CompoundTag> {
	private final LivingEntity ent;
	private BigDecimal energy_stored = new BigDecimal(0);
	private BigDecimal energy_stored_max = new BigDecimal(32000);

	public static final BigDecimal ENERGY_FOR_HUNGER_POINT = new BigDecimal(200);

	public AndroidCapability(LivingEntity ent) {
		this.ent = ent;
	}

	@Override
	@Nonnull
	public CompoundTag serializeNBT() {
		CompoundTag tag = new CompoundTag();
		tag.putString("energy_stored", energy_stored.toString());
		tag.putString("energy_stored_max", energy_stored_max.toString());
		return tag;
	}

	@Override
	public void deserializeNBT(CompoundTag compound) {
		if (compound.contains("energy_stored"))
			energy_stored = new BigDecimal(compound.getString("energy_stored"));

		if (compound.contains("energy_stored_max"))
			energy_stored_max = new BigDecimal(compound.getString("energy_stored_max"));
	}

	@Override
	public void tick() {
		if (ent instanceof ServerPlayer) {
			FoodData stats = ((ServerPlayer) ent).getFoodData();

			if (stats.needsFood()) {
				int food_points_missing = 20 - stats.getFoodLevel();
				boolean food_changed = false;
				float saturation_points_missing = 20F - stats.getSaturationLevel();
				boolean saturation_changed = false;

				while (food_points_missing > 0) {
					if (this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, true).equals(ENERGY_FOR_HUNGER_POINT)) {
						this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, false);
						food_points_missing--;
						food_changed = true;
					} else {
						break;
					}
				}

				while (saturation_points_missing >= 1F) {
					if (this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, true).equals(ENERGY_FOR_HUNGER_POINT)) {
						this.extractEnergyInner(ENERGY_FOR_HUNGER_POINT, false);
						saturation_points_missing--;
						saturation_changed = true;
					} else {
						break;
					}
				}

				if (food_changed) {
					stats.setFoodLevel(20 - food_points_missing);
				}

				if (saturation_changed) {
					stats.setSaturation(20F - saturation_points_missing);
				}

				OverdriveThatMatters.LOGGER.info("New power level {}", getBatteryLevel());
			}
		}
	}

	@Nonnull
	@Override
	public BigDecimal extractEnergyOuter(BigDecimal howMuch, boolean simulate) {
		return BigDecimal.ZERO;
	}

	@Nonnull
	@Override
	public BigDecimal extractEnergyInner(BigDecimal howMuch, boolean simulate) {
		BigDecimal new_energy = energy_stored.subtract(howMuch, MatteryCapability.ROUND_RULES).max(BigDecimal.ZERO);
		BigDecimal diff = energy_stored.subtract(new_energy);

		if (!simulate) {
			energy_stored = new_energy;
		}

		return diff;
	}

	@Nonnull
	@Override
	public BigDecimal receiveEnergyOuter(BigDecimal howMuch, boolean simulate) {
		BigDecimal new_energy = energy_stored.add(howMuch, MatteryCapability.ROUND_RULES).min(energy_stored_max);
		BigDecimal diff = new_energy.subtract(energy_stored);

		if (!simulate) {
			energy_stored = new_energy;
		}

		return diff;
	}

	@Nonnull
	@Override
	public BigDecimal receiveEnergyInner(BigDecimal howMuch, boolean simulate) {
		return receiveEnergyOuter(howMuch, simulate);
	}

	@Override
	public LivingEntity getEntity() {
		return this.ent;
	}

	@Override
	@Nonnull
	public BigDecimal getBatteryLevel() {
		return energy_stored;
	}

	@Override
	@Nonnull
	public BigDecimal getMaxBatteryLevel() {
		return energy_stored_max;
	}

	@Nonnull
	@Override
	public <T> LazyOptional<T> getCapability(@Nonnull Capability<T> cap, @Nullable Direction side) {
		if (cap == MatteryCapability.ANDROID) {
			return LazyOptional.of(() -> this).cast();
		}

		return LazyOptional.empty();
	}
}
